@@ -15,6 +15,8 @@ from charset_normalizer.probe_coherence import ProbeCoherence, HashableCounter
 
 from charset_normalizer.encoding import is_multi_byte_encoding
 
+from charset_normalizer.probe_inherent_sign import any_specified_encoding
+
 from loguru import logger
 
 from hashlib import sha256
@@ -273,6 +275,14 @@ class CharsetNormalizerMatches:
         """
         self._matches = matches
 
+        if len(self) > 1 and 'utf_8' in self.could_be_from_charset:
+
+            u8_index = self.could_be_from_charset.index('utf_8')
+
+            # Put UTF_8 at the beginning if not already the case
+            if u8_index > 0:
+                self._matches.insert(1, self._matches.pop(u8_index))
+
     def __iter__(self):
         for elem in self._matches:
             yield elem
@@ -319,7 +329,7 @@ class CharsetNormalizerMatches:
         return b_
 
     @staticmethod
-    def from_bytes(sequences, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, explain=False):
+    def from_bytes(sequences, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, preemptive_behaviour=True, explain=False):
         """
         Take a sequence of bytes that could potentially be decoded to str and discard all obvious non supported
         charset encoding.
@@ -327,7 +337,8 @@ class CharsetNormalizerMatches:
         :param bytes sequences: Actual sequence of bytes to analyse
         :param float threshold: Maximum amount of chaos allowed on first pass
         :param int chunk_size: Size to extract and analyse in each step
-        :param int steps: Number of steps
+        :param int steps: Number of steps/block to extract from sequence
+        :param bool preemptive_behaviour: Determine if we should look into sequence (ASCII-Mode) for pre-defined encoding
         :param bool explain: Print on screen what is happening when searching for a match
         :param list[str] cp_isolation: Finite list of encoding to use when searching for a match
         :param list[str] cp_exclusion: Finite list of encoding to avoid when searching for a match
@@ -381,6 +392,13 @@ class CharsetNormalizerMatches:
         tested = set()
         matches = list()
 
+        specified_encoding = any_specified_encoding(sequences) if preemptive_behaviour is True else None
+
+        if specified_encoding is not None:
+            warn(
+                'Trying to detect encoding on a sequence that seems to declare a encoding ({}).'.format(specified_encoding)
+            )
+
         for support in supported:
 
             k, p = support
@@ -443,6 +461,11 @@ class CharsetNormalizerMatches:
             ratios = [el.ratio for el in measures]
             nb_gave_up = [el.gave_up is True or el.ratio >= threshold for el in measures].count(True)
 
+            if len(ratios) == 0:
+                logger.warning('{encoding} was excluded because no measure can be done on sequence. ',
+                               encoding=p)
+                continue
+
             chaos_means = statistics.mean(ratios)
             chaos_median = statistics.median(ratios)
             # chaos_min = min(ratios)
@@ -493,8 +516,16 @@ class CharsetNormalizerMatches:
                     cnm
                 )
 
+            if specified_encoding is not None and p == specified_encoding:
+                logger.info('{encoding} is most likely the one. '
+                            'Because it is specified in analysed byte sequence and '
+                            'initial test passed successfully. '
+                            'Disable this behaviour by setting preemptive_behaviour '
+                            'to False', encoding=specified_encoding)
+                return CharsetNormalizerMatches([cnm]) if any(fingerprint_tests) is False else CharsetNormalizerMatches([matches[fingerprint_tests.index(True)]])
+
             if (p == 'ascii' and chaos_median == 0.) or bom_available is True:
-                logger.info('{encoding} is the most likely the one. {bom_available}',
+                logger.info('{encoding} is most likely the one. {bom_available}',
                                    encoding=p,
                                    bom_available='BOM/SIG available' if bom_available else '')
 
@@ -503,13 +534,14 @@ class CharsetNormalizerMatches:
         return CharsetNormalizerMatches(matches)
 
     @staticmethod
-    def from_fp(fp, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, explain=False):
+    def from_fp(fp, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, preemptive_behaviour=True, explain=False):
         """
         :param io.BinaryIO fp:
         :param int steps:
         :param int chunk_size:
         :param float threshold:
         :param bool explain: Print on screen what is happening when searching for a match
+        :param bool preemptive_behaviour: Determine if we should look into sequence (ASCII-Mode) for pre-defined encoding
         :param list[str] cp_isolation: Finite list of encoding to use when searching for a match
         :param list[str] cp_exclusion: Finite list of encoding to avoid when searching for a match
         :return: List of potential matches
@@ -522,16 +554,18 @@ class CharsetNormalizerMatches:
             threshold,
             cp_isolation,
             cp_exclusion,
+            preemptive_behaviour,
             explain
         )
 
     @staticmethod
-    def from_path(path, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, explain=False):
+    def from_path(path, steps=10, chunk_size=512, threshold=0.20, cp_isolation=None, cp_exclusion=None, preemptive_behaviour=True, explain=False):
         """
         :param str path:
         :param int steps:
         :param int chunk_size:
         :param float threshold:
+        :param bool preemptive_behaviour: Determine if we should look into sequence (ASCII-Mode) for pre-defined encoding
         :param bool explain: Print on screen what is happening when searching for a match
         :param list[str] cp_isolation: Finite list of encoding to use when searching for a match
         :param list[str] cp_exclusion: Finite list of encoding to avoid when searching for a match
@@ -539,7 +573,7 @@ class CharsetNormalizerMatches:
         :rtype: CharsetNormalizerMatches
         """
         with open(path, 'rb') as fp:
-            return CharsetNormalizerMatches.from_fp(fp, steps, chunk_size, threshold, cp_isolation, cp_exclusion, explain)
+            return CharsetNormalizerMatches.from_fp(fp, steps, chunk_size, threshold, cp_isolation, cp_exclusion, preemptive_behaviour, explain)
 
     @cached_property
     def could_be_from_charset(self):
@@ -596,3 +630,9 @@ class CharsetNormalizerMatches:
         return CharsetNormalizerMatches(
             sorted_matches[:nb_lowest_ratio+1]
         )
+
+
+# Some aliases to CharsetNormalizerMatches, because it is too long for a class name.
+CharsetDetector = CharsetNormalizerMatches
+EncodingDetector = CharsetNormalizerMatches
+CharsetDoctor = CharsetNormalizerMatches
