@@ -5,6 +5,11 @@ from charset_normalizer import CharsetNormalizerMatches as CnM
 
 class TestBytes(unittest.TestCase):
 
+    def test_too_short_none(self):
+        self.assertIsNotNone(
+            CnM.from_bytes(b'\xfe\xff').best().first()
+        )
+
     def test_bom_detection(self):
         with self.subTest('GB18030 UNAVAILABLE SIG'):
             self.assertFalse(
